/***************************************************************************
 *   fqterm, a terminal emulator for both BBS and *nix.                    *
 *   Copyright (C) 2008 fqterm development group.                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.               *
 ***************************************************************************/

#ifndef FQTERM_SSH_TYPES_H
#define FQTERM_SSH_TYPES_H

#include <QGlobalStatic>
#if !defined(Q_OS_WIN32) && !defined(_OS_WIN32_)
#include <sys/types.h>
#else 
typedef unsigned char u_char;
typedef unsigned short u_short;
typedef unsigned int u_int;
typedef unsigned long u_long;

#ifdef QT3_SUPPORT
typedef Q_UINT32 u_int32_t;
typedef Q_UINT64 u_int64_t;
#else 
typedef quint32 u_int32_t;
typedef quint64 u_int64_t;
#endif // QT3_SUPPORT

#endif  // Q_OS_WIN32

#endif  // FQTERM_SSH_TYPES_H
